from .PostProcessorBase import PostProcessorBase


# post processor for general purpose
class GenPostProcessor (PostProcessorBase):

    # constructor
    def __init__(self,taskBufferIF,ddmIF):
        PostProcessorBase.__init__(self,taskBufferIF,ddmIF)
        self.failOnZeroOkFile = True

    # main
    def doPostProcess(self, taskSpec, tmpLog):
        try:
            # get DDM I/F
            ddmIF = self.ddmIF.getInterface(taskSpec.vo, taskSpec.cloud)
            # skip if DDM I/F is inactive
            if not ddmIF:
                tmpLog.info('skip due to inactive DDM I/F')
            else:
                # loop over all datasets
                for datasetSpec in taskSpec.datasetSpecList:
                    # only output and log datasets
                    if datasetSpec.type not in ['log', 'output']:
                        continue
                    tmpLog.info(
                            'freezing datasetID={}:Name={}'.format(datasetSpec.datasetID, datasetSpec.datasetName))
                    ddmIF.freezeDataset(datasetSpec.datasetName, ignoreUnknown=True)
        except Exception as e:
            tmpLog.warning('failed to freeze datasets with {}'.format(str(e)))
            return self.SC_FAILED
        try:
            self.doBasicPostProcess(taskSpec, tmpLog)
        except Exception as e:
            tmpLog.error('doBasicPostProcess failed with {}'.format(str(e)))
            return self.SC_FATAL
        return self.SC_SUCCEEDED



"""
Checking DB schema version for PanDA JEDI.
If there is an issue and the pandaEmailNotification var is
defined in panda_config, it will send an email notification.
"""

from pandajedi.jediconfig import jedi_config
from pandaserver.config import panda_config
from pandajedi.jedicore.JediDBProxy import DBProxy
from pandaserver.srvcore.MailUtils import MailUtils
from pandajedi.jedicore import JediDBSchemaInfo

from packaging import version

proxyS = DBProxy()
proxyS.connect(jedi_config.db.dbhost, jedi_config.db.dbpasswd, jedi_config.db.dbuser, jedi_config.db.dbname)

sql = "select major || '.' || minor || '.' || patch from ATLAS_PANDA.pandadb_version where component = 'JEDI'"

res = proxyS.querySQL(sql)
dbVersion = res[0][0]

serverDBVersion = JediDBSchemaInfo.JediDBSchemaInfo().method()

if version.parse(dbVersion) >= version.parse(serverDBVersion):
    print ("True")
else:
    print ("False")
    if 'pandaEmailNotification' in panda_config.__dict__:
        msgBody = 'There is an issue with ' + panda_config.pserveralias + '. PanDA DB schema installed is ' + dbVersion + \
                  ' while PanDA JEDI requires version ' + serverDBVersion + \
                  ' to be installed. Please check the official docs for instructions on how to upgrade the schema.'
        MailUtils().send(panda_config.pandaEmailNotification,
                         'PanDA DB Version installed is not correct for JEDI running on ' + panda_config.pserveralias, msgBody)



from pandajedi.jedicore.MsgWrapper import MsgWrapper
from .JobThrottlerBase import JobThrottlerBase

# logger
from pandacommon.pandalogger.PandaLogger import PandaLogger
logger = PandaLogger().getLogger(__name__.split('.')[-1])

# class to throttle ATLAS production jobs
class AtlasProdJobThrottler (JobThrottlerBase):

    # constructor
    def __init__(self,taskBufferIF):
        JobThrottlerBase.__init__(self, taskBufferIF)
        self.comp_name = 'prod_job_throttler'
        self.app = 'jedi'

    # check if throttled
    def toBeThrottled(self, vo, prodSourceLabel, cloudName, workQueue, resource_name):
        return self.toBeThrottledBase(vo, prodSourceLabel, cloudName, workQueue, resource_name, logger)



"""
Minimum schema version required for panda-jedi to work properly.
Please always keep this version number up to date.
"""

from pandacommon.pandalogger.PandaLogger import PandaLogger

# logger
_logger = PandaLogger().getLogger('DBSchema')

class JediDBSchemaInfo():
    schema_version = None
    def method(self):
        schema_version = '0.0.12'
        _logger.debug("PanDA schema version required for JEDI is : %s" % schema_version)
        return (schema_version)



import multiprocessing

from pandajedi.jediorder import JediMsgProcessor


stop_event = multiprocessing.Event()

agent = multiprocessing.Process(target=JediMsgProcessor.launcher,
                                args=(stop_event,))
agent.start()
stop_event.wait()



from .JobThrottlerBase import JobThrottlerBase

# logger
from pandacommon.pandalogger.PandaLogger import PandaLogger

logger = PandaLogger().getLogger(__name__.split('.')[-1])


# class to throttle ATLAS analysis jobs
class AtlasAnalJobThrottler(JobThrottlerBase):

    # constructor
    def __init__(self, taskBufferIF):
        JobThrottlerBase.__init__(self, taskBufferIF)
        self.logger_name = __name__.split('.')[-1]
        self.comp_name = 'anal_job_throttler'
        self.app = 'jedi'

    # check if throttled
    def toBeThrottled(self, vo, prodSourceLabel, cloudName, workQueue, resource_name):
        return self.toBeThrottledBase(vo, prodSourceLabel, cloudName, workQueue, resource_name, logger)

    # check if throttled
    def toBeThrottled_old(self, vo, prodSourceLabel, cloudName, workQueue, resource_name):
        # make logger
        tmpLog = MsgWrapper(logger)
        tmp_log.debug('start vo={0} label={1} cloud={2} workQueue={3}'.format(vo, prodSourceLabel, cloudName,
                                                                             workQueue.queue_name))

        # check if unthrottled
        if not workQueue.throttled:
            tmpLog.debug("  done : unthrottled since throttled is False")
            return self.retUnThrottled

        tmpLog.debug("  done : SKIP")
        return self.retThrottled
